from src.datasets_metadata.table.datasetes_metadata_csv import DatasetMetadataSCV
from src.datasets_metadata.dict.datasetes_metadata_pickle import DatasetMetadataPickle

__author__ = "Lion Ben Nedava"
__email__ = "lionben89@gmail.com"
__version__ = "1.0.0"


def get_module_version():
    return __version__


__all__ = ["DatasetMetadataSCV", "DatasetMetadataPickle"]



import aicsimageio
import numpy as np
import logging
import typing
import tifffile
from aicsimageio import AICSImage, aics_image
from multipledispatch import dispatch


log = logging.getLogger(__name__)

"""
ImageUtils
------------------
Class that contains some static methods to help handle images
Channel= 'C'
MosaicTile= 'M'
Samples= 'S'
SpatialX= 'X'
SpatialY= 'Y'
SpatialZ= 'Z'
Time= 'T'

Two main functions params are
image - AICSImage ("STCZYX")
image_ndarray - np.ndarray ("CZYX")
"""


class ImageUtils:

    @staticmethod
    def imread(image: typing.Union[np.ndarray, str]) -> AICSImage:
        with AICSImage(image) as img:
            img.size_z
            return img

    @staticmethod
    def imsave(image_ndarray: np.ndarray, path: str):
        tifffile.imsave(path, image_ndarray, image_ndarray.shape)

    @staticmethod
    @dispatch(AICSImage, int)
    def get_channel(image: AICSImage, channel_index: int) -> np.ndarray:
        return image.get_image_data("CZYX")[channel_index:channel_index+1, :, :, :]

    @staticmethod
    @dispatch(np.ndarray, int)
    def get_channel(image_ndarray: np.ndarray, channel_index: int) -> np.ndarray:
        return image_ndarray[channel_index:channel_index+1, :, :, :]

    @staticmethod
    def add_channel(image_ndarray: np.ndarray, channel) -> np.ndarray:
        new_image_ndarray = np.append(image_ndarray, channel, axis=0)
        return new_image_ndarray

    @staticmethod
    def image_to_ndarray(image):
        return image.get_image_data("CZYX")

    @staticmethod
    def get_channel_names(image):
        return image.get_channel_names()

    """
    Normalize all values between 0 to 1
    """
    @staticmethod
    def normalize(image_ndarray) -> np.ndarray:
        return image_ndarray/np.max(image_ndarray)



import logging
import typing
from abc import ABC, abstractmethod
import pandas as pd
from multipledispatch import dispatch

from numpy import number

log = logging.getLogger(__name__)

"""
DatasetsMetaDataAbstract
------------------
Abstract method th handle datasets metadata
the data is handled as DataFrame of pandas


"""

class DatasetsMetaDataAbstract:
     
     def __init__(self,destenation,source=None) -> None:
         self.source:str = source
         self.destenation:str = destenation
         self.data:pd.DataFrame = None
     
     @abstractmethod
     def create(self):
          pass
     
     def get_data(self)->pd.DataFrame:
          return self.data
    
     
     
     
     
     
     
     


# from src.contact_sites_provider import ContactSitesProvider
# from src.contact_sites_result import ContactSitesResult

__author__ = "Lion Ben Nedava"
__email__ = "lionben89@gmail.com"
__version__ = "1.0.0"


def get_module_version():
    return __version__


# __all__ = ["ContactSitesProvider", "ContactSitesResult"]



import os
import logging

from src.image.image_utils import ImageUtils

log = logging.getLogger(__name__)
images_save_dir = "{}\\tests\\images".format(os.getcwd())
results_save_dir = "{}\\tests\\results".format(os.getcwd())


organelle_name = "Nuclear_envelop"
image_file_name = "29014c3c_3500000883_100X_20170509_F06_P43.ome.tiff"
result_image_file_name = "test_29014c3c_3500000883_100X_20170509_F06_P43.ome.tiff"
# seg_image_file_name = "image_list_test.csv"

if not os.path.exists(results_save_dir):
    os.makedirs(results_save_dir)

def test_image_utils() -> None:
    image = ImageUtils.imread("{}\\{}\\{}".format(images_save_dir,organelle_name,image_file_name))
    print(ImageUtils.get_channel_names(image))
    image_ndarray = ImageUtils.normalize(ImageUtils.image_to_ndarray(image))
    bf_image = ImageUtils.get_channel(image,3)
    fl_image = ImageUtils.get_channel(image_ndarray,2)
    n_image = ImageUtils.add_channel(bf_image,fl_image)
    print(n_image.shape)
    # ImageUtils.imsave(n_image,"{}\\{}\\{}".format(images_save_dir,organelle_name,result_image_file_name))
    test_image = ImageUtils.imread(n_image)
    ImageUtils.imsave(ImageUtils.image_to_ndarray(test_image),"{}\\{}\\{}".format(images_save_dir,organelle_name,result_image_file_name))

    return None


test_image_utils()


import setuptools

__version__ = '1.0.6'

with open("README.md", "r") as fh:
    readme = fh.read()

requirements = [
    "matplotlib",
    "numpy",
    "tifffile",
    "scikit-image>=0.15.0",
    "aicsimageio==3.0.7",
    "quilt3",
    "bumpversion",
    "twine",
    "setuptools>=42",
    "wheel",
    "pandas",
    "multipledispatch"
]


setuptools.setup(
    author="Lion Ben Nedava",
    author_email="lionben89@gmail.com",
    install_requires=requirements,
    license="MIT",
    long_description=readme,
    long_description_content_type="text/markdown",
    include_package_data=True,
    keywords="cell_imaging_utils",
    name="cell_imaging_utils",
    packages=setuptools.find_packages(exclude=["images"]),
    python_requires=">=3.6",
    test_suite="tests",
    url="https://github.com/lionben89/BGU_cell_imaging_utils",
    # Do not edit this string manually, always use bumpversion
    # Details in CONTRIBUTING.rst
    version=__version__,
    zip_safe=False
)



import os
import logging

from src.datasets_metadata.table.datasetes_metadata_csv import DatasetMetadataSCV
from src.datasets_metadata.dict.datasetes_metadata_pickle import DatasetMetadataPickle
from src.image.image_utils import ImageUtils

log = logging.getLogger(__name__)
csvs_save_dir = "{}\\tests\\csvs".format(os.getcwd())
pickle_save_dir = "{}\\tests\\pickles".format(os.getcwd())
results_save_dir = "{}\\tests\\results".format(os.getcwd())


organelle_name = "Nuclear_envelop"
train_file_name = "image_list_train.csv"
test_file_name = "image_list_test.csv"
pickle_file_name = "bestResults.p"

if not os.path.exists(results_save_dir):
    os.makedirs(results_save_dir)

def test_datasets_metadata() -> None:
    image_list_train = DatasetMetadataSCV("{}\\{}\\{}".format(csvs_save_dir,organelle_name,train_file_name),"{}\\{}\\{}".format(csvs_save_dir,organelle_name,train_file_name))
    seg_image_list_train = DatasetMetadataSCV("{}\\{}\\{}".format(results_save_dir,organelle_name,train_file_name))
    pickle_best_res = DatasetMetadataPickle("{}\\{}\\{}".format(pickle_save_dir,organelle_name,train_file_name),"{}\\{}".format(pickle_save_dir,pickle_file_name))
    print(pickle_best_res.get_data())
    print(seg_image_list_train.get_data())
    
    return None


test_datasets_metadata()